use std::fmt::{Display, Formatter};

pub use VkResult::*;

#[repr(u32)]
#[derive(Clone, Copy, Debug, Eq, PartialEq)]
pub enum VkResult {
    VK_SUCCESS = 0,
    VK_NOT_READY = 1,
    VK_TIMEOUT = 2,
    VK_EVENT_SET = 3,
    VK_EVENT_RESET = 4,
    VK_INCOMPLETE = 5,
    VK_ERROR_OUT_OF_HOST_MEMORY = -1i32 as u32,
    VK_ERROR_OUT_OF_DEVICE_MEMORY = -2i32 as u32,
    VK_ERROR_INITIALIZATION_FAILED = -3i32 as u32,
    VK_ERROR_DEVICE_LOST = -4i32 as u32,
    VK_ERROR_MEMORY_MAP_FAILED = -5i32 as u32,
    VK_ERROR_LAYER_NOT_PRESENT = -6i32 as u32,
    VK_ERROR_EXTENSION_NOT_PRESENT = -7i32 as u32,
    VK_ERROR_FEATURE_NOT_PRESENT = -8i32 as u32,
    VK_ERROR_INCOMPATIBLE_DRIVER = -9i32 as u32,
    VK_ERROR_TOO_MANY_OBJECTS = -10i32 as u32,
    VK_ERROR_FORMAT_NOT_SUPPORTED = -11i32 as u32,
    VK_ERROR_FRAGMENTED_POOL = -12i32 as u32,
    VK_ERROR_OUT_OF_POOL_MEMORY = -1_000_069_000i32 as u32,
    VK_ERROR_INVALID_EXTERNAL_HANDLE = -1_000_072_003i32 as u32,
    VK_ERROR_SURFACE_LOST_KHR = -1_000_000_000i32 as u32,
    VK_ERROR_NATIVE_WINDOW_IN_USE_KHR = -1_000_000_001i32 as u32,
    VK_SUBOPTIMAL_KHR = 1_000_001_003i32 as u32,
    VK_ERROR_OUT_OF_DATE_KHR = -1_000_001_004i32 as u32,
    VK_ERROR_INCOMPATIBLE_DISPLAY_KHR = -1_000_003_001i32 as u32,
    VK_ERROR_VALIDATION_FAILED_EXT = -1_000_011_001i32 as u32,
    VK_ERROR_INVALID_SHADER_NV = -1_000_012_000i32 as u32,
    VK_ERROR_INVALID_DRM_FORMAT_MODIFIER_PLANE_LAYOUT_EXT = -1_000_158_000i32 as u32,
    VK_ERROR_FRAGMENTATION_EXT = -1_000_161_000i32 as u32,
    VK_ERROR_NOT_PERMITTED_EXT = -1_000_174_001i32 as u32,
    VK_ERROR_INVALID_DEVICE_ADDRESS_EXT = -1_000_244_000i32 as u32,
    VK_ERROR_FULL_SCREEN_EXCLUSIVE_MODE_LOST_EXT = -1_000_255_000i32 as u32,
    VK_THREAD_IDLE_KHR = 1_000_268_000,
    VK_THREAD_DONE_KHR = 1_000_268_001,
    VK_OPERATION_DEFERRED_KHR = 1_000_268_002,
    VK_OPERATION_NOT_DEFERRED_KHR = 1_000_268_003,
    VK_ERROR_PIPELINE_COMPILE_REQUIRED_EXT = 1_000_297_000,
}

impl VkResult {
    pub fn as_str(&self) -> &str {
        match self {
            VK_SUCCESS => "VK_SUCCESS",
            VK_NOT_READY => "VK_NOT_READY",
            VK_TIMEOUT => "VK_TIMEOUT",
            VK_EVENT_SET => "VK_EVENT_SET",
            VK_EVENT_RESET => "VK_EVENT_RESET",
            VK_INCOMPLETE => "VK_INCOMPLETE",
            VK_ERROR_OUT_OF_HOST_MEMORY => "VK_ERROR_OUT_OF_HOST_MEMORY",
            VK_ERROR_OUT_OF_DEVICE_MEMORY => "VK_ERROR_OUT_OF_DEVICE_MEMORY",
            VK_ERROR_INITIALIZATION_FAILED => "VK_ERROR_INITIALIZATION_FAILED",
            VK_ERROR_DEVICE_LOST => "VK_ERROR_DEVICE_LOST",
            VK_ERROR_MEMORY_MAP_FAILED => "VK_ERROR_MEMORY_MAP_FAILED",
            VK_ERROR_LAYER_NOT_PRESENT => "VK_ERROR_LAYER_NOT_PRESENT",
            VK_ERROR_EXTENSION_NOT_PRESENT => "VK_ERROR_EXTENSION_NOT_PRESENT",
            VK_ERROR_FEATURE_NOT_PRESENT => "VK_ERROR_FEATURE_NOT_PRESENT",
            VK_ERROR_INCOMPATIBLE_DRIVER => "VK_ERROR_INCOMPATIBLE_DRIVER",
            VK_ERROR_TOO_MANY_OBJECTS => "VK_ERROR_TOO_MANY_OBJECTS",
            VK_ERROR_FORMAT_NOT_SUPPORTED => "VK_ERROR_FORMAT_NOT_SUPPORTED",
            VK_ERROR_FRAGMENTED_POOL => "VK_ERROR_FRAGMENTED_POOL",
            VK_ERROR_OUT_OF_POOL_MEMORY => "VK_ERROR_OUT_OF_POOL_MEMORY",
            VK_ERROR_INVALID_EXTERNAL_HANDLE => "VK_ERROR_INVALID_EXTERNAL_HANDLE",
            VK_ERROR_SURFACE_LOST_KHR => "VK_ERROR_SURFACE_LOST_KHR",
            VK_ERROR_NATIVE_WINDOW_IN_USE_KHR => "VK_ERROR_NATIVE_WINDOW_IN_USE_KHR",
            VK_SUBOPTIMAL_KHR => "VK_SUBOPTIMAL_KHR",
            VK_ERROR_OUT_OF_DATE_KHR => "VK_ERROR_OUT_OF_DATE_KHR",
            VK_ERROR_INCOMPATIBLE_DISPLAY_KHR => "VK_ERROR_INCOMPATIBLE_DISPLAY_KHR",
            VK_ERROR_VALIDATION_FAILED_EXT => "VK_ERROR_VALIDATION_FAILED_EXT",
            VK_ERROR_INVALID_SHADER_NV => "VK_ERROR_INVALID_SHADER_NV",
            VK_ERROR_INVALID_DRM_FORMAT_MODIFIER_PLANE_LAYOUT_EXT => {
                "VK_ERROR_INVALID_DRM_FORMAT_MODIFIER_PLANE_LAYOUT_EXT"
            }
            VK_ERROR_FRAGMENTATION_EXT => "VK_ERROR_FRAGMENTATION_EXT",
            VK_ERROR_NOT_PERMITTED_EXT => "VK_ERROR_NOT_PERMITTED_EXT",
            VK_ERROR_INVALID_DEVICE_ADDRESS_EXT => "VK_ERROR_INVALID_DEVICE_ADDRESS_EXT",
            VK_ERROR_FULL_SCREEN_EXCLUSIVE_MODE_LOST_EXT => {
                "VK_ERROR_FULL_SCREEN_EXCLUSIVE_MODE_LOST_EXT"
            }
            VK_THREAD_IDLE_KHR => "VK_THREAD_IDLE_KHR",
            VK_THREAD_DONE_KHR => "VK_THREAD_DONE_KHR",
            VK_OPERATION_DEFERRED_KHR => "VK_OPERATION_DEFERRED_KHR",
            VK_OPERATION_NOT_DEFERRED_KHR => "VK_OPERATION_NOT_DEFERRED_KHR",
            VK_ERROR_PIPELINE_COMPILE_REQUIRED_EXT => "VK_ERROR_PIPELINE_COMPILE_REQUIRED_EXT",
        }
    }
}

impl Display for VkResult {
    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
        write!(f, "Result from a vulkan api call ({})", self.as_str())
    }
}

impl std::error::Error for VkResult {}
